_MAPS_DIR = "../rsc/maps/"

_O2J_DIR = _MAPS_DIR + "o2jam/"
_OSU_DIR = _MAPS_DIR + "osu/"
_QUA_DIR = _MAPS_DIR + "qua/"
_SM_DIR = _MAPS_DIR + "sm/"

O2J_CHECK_IT_OUT_OJN = _O2J_DIR + "o2ma120.ojn"
O2J_CHECK_IT_OUT_OJM = _O2J_DIR + "o2ma120.ojm "
O2J_FLY_MAGPIE_OJN = _O2J_DIR + "o2ma178.ojn"
O2J_FLY_MAGPIE_OJM = _O2J_DIR + "o2ma178.ojm"

OSU_ADDICTION_CUT = _OSU_DIR + "AddictionCut.osu"  # Small file to test quickly
OSU_AVENGER_HITSOUNDABLE = _OSU_DIR + "AvengerHitsoundable.osu"
OSU_AVENGER_HITSOUNDFILE = _OSU_DIR + "AvengerHitsoundFile.osu"
OSU_CARAVAN = _OSU_DIR + "Caravan.osu"
OSU_CHECK_IT_OUT = _OSU_DIR + "CheckItOut.osu"
OSU_DOPPELGANGER = _OSU_DIR + "DoppelGanger.osu"
OSU_ESCAPES = _OSU_DIR + "Escapes.osu"
OSU_FOOL_MOON_NIGHT = _OSU_DIR + "FoolMoonNight.osu"
OSU_GRAVITY = _OSU_DIR + "Gravity.osu"
OSU_ICFITU = _OSU_DIR + "ICFITU.osu"
OSU_JOUNETSU = _OSU_DIR + "JounetsuTairiku.osu"
OSU_LAST_RESORT = _OSU_DIR + "LastResort.osu"
OSU_LEG = _OSU_DIR + "LEG.osu"
OSU_LN_DAN_14 = _OSU_DIR + "LNDan14.osu"
OSU_LN_DAN_14 = _OSU_DIR + "LNDan15.osu"
OSU_PLANET_SHAPER = _OSU_DIR + "PLANETSHAPER.osu"
OSU_REBIRTH = _OSU_DIR + "Rebirth.osu"
OSU_TRIBAL_TRIAL_EXH = _OSU_DIR + "TribalTrialEXH.osu"
OSU_TRIBAL_TRIAL_MX = _OSU_DIR + "TribalTrialMX.osu"
OSU_WORLD_FRAGMENTS = _OSU_DIR + "WorldFragments.osu"
OSU_ZENITHALIZE_19 = _OSU_DIR + "ZENITHALIZE_19.osu"

QUA_NEURO_CLOUD = _QUA_DIR + "NeuroCloud.qua"

SM_CARAVAN = _SM_DIR + "Caravan.sm"
SM_ESCAPES = _SM_DIR + "Escapes.sm"
SM_GRAVITY = _SM_DIR + "Gravity.sm"
SM_ICFITU = _SM_DIR + "ICFITU.sm"
