import unittest


class MyTestCase(unittest.TestCase):

    """ Not Implemented
    This shouldn't be test-able since it's a base abstract class
    However there's plan to create a universal format, will how that goes.
    For now this is just placeholder
    """

if __name__ == '__main__':
    unittest.main()
