
class SMConst:
    HIT_STRING: str = "1"
    HOLD_STRING_HEAD: str = "2"
    HOLD_STRING_TAIL: str = "3"
    LIFT_STRING: str = "L"
    KEYSOUND_STRING: str = "K"
    FAKE_STRING: str = "F"
    MINE_STRING: str = "M"
    ROLL_STRING_HEAD: str = "4"
    ROLL_STRING_TAIL: str = "3"
