from dataclasses import dataclass
from reamber.base.BpmObj import BpmObj


@dataclass
class O2JBpmObj(BpmObj):
    """ Defines the O2Jam Bpm Object

    The O2Jam Bpm Object is stored in binary file .ojn
    """
    pass
